// grupa.h - Klasa grupe električnih potrošača.

#ifndef _grupa_h_
#define _grupa_h_

#include "potrosac1.h"
#include <iostream>
#include <utility>
using std::move;

namespace Potrosaci {
  class Grupa: public Potrosac {
    struct Elem {                          // Element liste potrošača.
      Potrosac* potr; Elem* sled;
      Elem(Potrosac* p): potr{p}, sled{nullptr} {}
      ~Elem() { delete potr; }
    };
    Elem *prvi, *posl;                     // Prvi i poslednji element.
    double sna;                            // Snaga grupe.
    void kopiraj(const Grupa& g);          // Kopiranje u grupu.
    void premesti(Grupa& g) {              // Premeštanje u grupu.
      prvi = g.prvi; posl = g.posl; sna = g.sna;
      g.prvi = g.posl = nullptr;
    }
    void brisi();                          // Oslobađanje memorije.
    void pisi(ostream& it) const override; // Pisanje grupe.
  public:
    Grupa(): prvi{nullptr}, posl{nullptr}{}// Stvaranje prazne grupe.
    Grupa(const Grupa& g) { kopiraj(g); }  // Kopirajući konstruktor.
    Grupa(Grupa&& g) { premesti(g); }      // Premeštajući konstruktor.
    ~Grupa() { brisi(); }                  // Destruktor.
    Grupa& operator=(const Grupa& g) {     // Kopirajuća dodela vrednosti.
      if (this != &g) { brisi(); kopiraj(g); }
      return *this;
    }
    Grupa& operator=(Grupa&& g) {          // Premeštajuća dodela vrednosti.
      if (this != &g) { brisi(); premesti(g); }
      return *this;
    }
    Grupa& operator+=(const Potrosac& p) { // Dodavanje potrošača.
      Elem* novi = new Elem(p.kopija());
      posl = (!prvi ? prvi : posl->sled) = novi;
      sna += p.snaga(); return *this;
    }
    double snaga() const override { return sna; }    // Snaga grupe.
    Grupa* kopija() const& override        // Kopija grupe kopiranjem.
      { return new Grupa(*this); }
    Grupa* kopija() && override            // Kopija grupe premeštanjem.
      { return new Grupa(move(*this)); }
  }; // class Grupa
} // namespace Potrosaci

#endif
