// niz6.h - Generička klasa nizova pokazivača na elemente.

#ifndef _niz6_h_
#define _niz6_h_

#include <iostream>
using std::ostream;

class G_indeks {};             // KLASA GREŠAKA: Nedozvoljen indeks.
inline ostream& operator<<(ostream& it, const G_indeks&)
  { return it << "*** Nedozvoljen indeks! ***"; }

class G_pun {};                // KLASA GREŠAKA: Niz je pun.
inline ostream& operator<<(ostream& it, const G_pun&)
  { return it << "*** Niz je pun! ***"; }

template <typename T>                    // GENERIČKA KLASA NIZOVA:
class Niz {
  T* niz;                                // Niz elemenata.
  size_t kap, duz;                       // Kapacitet i dužina niza.
public:
  explicit Niz(size_t kap=10)            // Stvaranje praznog niza.
    : niz{new T[kap]}, kap{kap}, duz{0} {}
  Niz(const Niz& n) { kopiraj(n); }      // Kopirajući konstruktor.
  Niz(Niz&& n) { premesti(n); }          // Premeštajući konstruktor.
  ~Niz() { brisi(); }                    // Destruktor.
  Niz& operator=(const Niz& n) {         // Kopirajuća dodela vrednosti.
    if (this != &n) { brisi(); kopiraj(n); }
    return *this;
  }
  Niz& operator=(Niz&& n) {              // Premeštajuća dodela vrednosti.
    if (this != &n) { brisi(); premesti(n); }
    return *this;
  }
  size_t duzina() const { return duz; }  // Dužina niza.
  Niz& operator+=(const T& t) {          // Dodavanje elementa.
    if (duz == kap) throw G_pun();
    niz[duz++] = t;
    return *this;
  }
  T& operator[](size_t i) {              // Pristup elementu
    if (i<0 || i>=duz) throw G_indeks(); // - promenljivog niza,
    return niz[i];
  }
  const T& operator[](size_t i) const {  // - nepromenljivog niza.
    if (i<0 || i>=duz) throw G_indeks();
    return niz[i];
  }
private:
  void kopiraj(const Niz& n) {           // Kopiranje u niz.
    niz = new T[kap = n.kap];
    duz = n.duz;
    for (size_t i=0; i<duz; i++) niz[i] = n.niz[i];
  }
  void premesti(Niz& n) {                // Premeštanje u niz.
    niz = n.niz; kap = n.kap; duz = n.duz;
    n.niz = nullptr;
  }
  void brisi() { delete[] niz; }         // Oslobađanje memorije.
};

#endif
