// pokaz.h - Klasa pametnih pokazivača.

#ifndef _pokaz_h_
#define _pokaz_h_

template<typename T>
class Pokaz {
  struct Pok {                             // Kontrolni blok:
    T* pod;                                // - pokazivani podatak,
    size_t br;                             // - broj pokazivača,
    Pok(T* pod): br{1}, pod{pod} {}        // - konstruktor.
  };
  Pok* pok;                                // Pokazivač na konstrolni blok.
public:
  Pokaz(): pok{nullptr} {}                 // Stvaranje praznog pokazivača.
  Pokaz(T* pod): pok{new Pok{pod}} {}      // Stvaranje s podatkom.
  Pokaz(const Pokaz& pokaz)                // Kopirajući konstruktor.
    { kopiraj(pokaz); }
  Pokaz(Pokaz&& pokaz) { premesti(pokaz); }// Premeštajući konstruktor.
  ~Pokaz() { brisi();  }                   // Destruktor.
  Pokaz& operator=(const Pokaz& pokaz) {   // Kopirajuća dodela vrednosti.
    if (this != &pokaz) { brisi(); kopiraj(pokaz); }
    return *this;
  }
  Pokaz& operator=(Pokaz&& pokaz) {        // Premeštajuća dodela vrednosti
    if (this != &pokaz) { brisi(); premesti(pokaz); }
    return *this;
  }
  T& operator*() { return *pok->pod; }     // Dohvatanje podatka.
  const T& operator*() const { return *pok->pod; }
  T* operator->() { return pok->pod; }     // Dohvatanje adrese podatka.
  const T* operator->() const { return pok->pod; }
  friend bool operator==(const Pokaz& p1, const Pokaz& p2) {
      return p1.pok->pod == p2.pok->pod;   // Da li su jednaki?
  }
private:
  void kopiraj(const Pokaz& pokaz) {       // Kopiranje u objekat.
    pok = pokaz.pok;
    if (pok) ++pok->br;
  }
  void premesti(Pokaz& pokaz) {            // Premeštanje objekta.
      pok = pokaz.pok;
      pokaz.pok = nullptr;
  }
  void brisi() {                           // Oslobađanje memorije.
    if (pok) {
      if (--pok->br == 0) {
        delete pok->pod;
        delete pok;
        pok = nullptr;
      }
    }
  }
};

#endif
