// tekst2.h – Štedljiva klasa tekstova.

#include <iostream>
#include <cstring>
#include <cstdlib>
using std::istream, std::ostream, std::strlen;

class Tekst {
  struct Tks {              // Element liste svih tekstova:
    int   kor;              // - broj korišćenja teksta,
    Tks  *sled, *pret;      // - pokazivači na sledeći i prethodni tekst,
    char* niz;              // - pokazivač na nisku.
  };
  static inline Tks* prvi{nullptr};   // Pokazivač na prvi tekst.
  static inline Tks* posl{nullptr};   // Pokazivač na poslednji tekst.
  static inline int broj{0};          // Ukupan broj tekstova.

  Tks* tks;                                   // Pokazivač na tekst.
  void kopiraj(const Tekst& t)                // Kopiranje u tekst.
    { if ((tks = t.tks) != nullptr) tks->kor++; }
  void brisi();                               // Oslobađanje memorije.
public:                                       // Inicijalizacija:
  Tekst(): tks{nullptr} {}                    // - kao prazan tekst,
  Tekst(const char*);                         // - nizom znakova,
  Tekst(const Tekst& t) { kopiraj(t); }       // - drugim tekstom.
  ~Tekst() { brisi(); }                       // Uništavanje.
  Tekst& operator=(const Tekst& t) {          // Dodela vrednosti.
    if (this != &t) { brisi(); kopiraj(t); }
    return *this;
  }
  friend int len(const Tekst& t) // Dužina teksta.
    { return t.tks ? strlen(t.tks->niz) : 0; }
 
  friend Tekst operator+                      // Spajanje tekstova.
    (const Tekst&, const Tekst&);
  Tekst& operator+=(const Tekst& t)           // Dodavanje teksta.
    { return *this = *this + t; }
  const char& operator[](int i) const {       // Znak u tekstu
    if (i<0 || len(*this)<=i) exit(1);        //   (ne može da se promeni).
    return tks->niz[i];
  }
  Tekst operator()(int, int) const;                      // Podniz teksta.
  friend int operator%(const Tekst&, const Tekst&);      // Mesto podniza.
  friend ostream& operator<<(ostream& it, const Tekst& t)// Pisanje teksta.
    { if (t.tks) it << t.tks->niz; return it; }
  friend istream& operator>>(istream& ut, Tekst& t)      // Čitanje teksta.
    { char n[81]; ut >> n; if (ut) t = n; return ut; }
  static int ukupno() { return broj; }        // Ukupan broj tekstova.
  static void spisak();                       // Spisak svih tekstova.
};
